# -*- coding: utf-8 -*-

import tempfile
import os

from queue import Queue
from tools import trace, timer

from View.ASubWindow import ASubWindow, ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect, QTimer, QProcess,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
    QTextEdit,
)

_translate = QCoreApplication.translate

class SolverLogFileWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Solver logs",
                 file_name = None,
                 study=None, config=None,
                 solver=None, parent=None):
        self._title = title
        self._parent = parent

        self._study = study
        self._config = config
        self._solver = solver

        self._file_name = file_name

        super(SolverLogFileWindow, self).__init__(
            name=self._title, ui="SolverLogFile", parent=parent
        )
        self.ui.setWindowTitle(self._title)

        self.setup_action()
        self.setup_connections()
        self.setup_text()

    def setup_action(self):
        self.find(QAction, "action_revert").setEnabled(True)
        if self._config.editor != "":
            self.find(QAction, "action_open_in_editor").setEnabled(True)
        else:
            self.find(QAction, "action_open_in_editor").setEnabled(False)

    def setup_connections(self):
        self.find(QAction, "action_revert").triggered.connect(self.revert)
        self.find(QAction, "action_open_in_editor").triggered.connect(self.open_on_editor)

    def setup_text(self):
        with open(self._file_name, "r") as f:
            for line in f:
                line = line.rstrip()
                self.find(QTextEdit, "textEdit").append(line)

    def revert(self):
        self.find(QTextEdit, "textEdit").clear()
        self.setup_text()

    def open_on_editor(self):
        p = QProcess(self)

        cmd = self._config.editor
        cmd = cmd.replace("@file", self._file_name)

        cmd = cmd.split()
        exe = cmd[0]
        args = cmd[1:]

        p.start(
            exe, args,
        )
